package com.expoware;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.os.Vibrator;
import android.text.Html;
import android.view.*;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

public class GameActivity extends Activity
{
    private Game currentGame;

    // References to UI widgets in main.xml
    public TextView PlayerLabelRef;
    public TextView LevelLabelRef;
    public TextView AttemptsLabelRef;
    public TextView HintsLabelRef;
    public TextView StatusLabelRef;
    public EditText NumberToTryRef;
    public Button TryButtonRef;
    public View.OnClickListener TryButtonClick;

    @Override
    public void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        setContentView(R.layout.game);

        // Complete initialization
        GameActivityInitializer.PrepareWidgets(this);

        // Retrieve passed values and arrange a new Game object
        String playerName = getIntent().getExtras().getString(Constants.PlayerNameKey);
        GameLevel level = GameLevel.Basic;
        int index = getIntent().getIntExtra(Constants.GameLevelKey, -1);
        if(index >= 0 && index < GameLevel.values().length)
            level = GameLevel.values()[index];

        currentGame = new Game(playerName, level);
        RefreshView();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu)
    {
        // Invoked only once. Only use for a one-off initialization.
        return true;
    }

    @Override
    public boolean onPrepareOptionsMenu(Menu menu)
    {
        // Clear the current menu
        menu.clear();

        // Determine the menu to load based on the context
        int menuResourceId = R.menu.game;
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(menuResourceId, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item)
    {
        // Handle item selection
        switch (item.getItemId())
        {
            case R.id.menuQuit:
                QuitGame();
                return true;
            case R.id.menuHint:
                ShowHint();
                return true;
            default:
                return super.onOptionsItemSelected(item);
        }
    }

    public void NewAttempt(int number)
    {
        boolean isValid = currentGame.isValidAttempt(number);
        if (!isValid)
        {
            // Toast and vibrate
            GameActivityInitializer.Toast(getString(R.string.InvalidAttempt));
            Vibrator v = (Vibrator) getSystemService(Context.VIBRATOR_SERVICE);
            v.vibrate(300);     // milliseconds
            return;
        }
        boolean success = currentGame.makeAttempt(number);
        if (!success)
        {
            UpdateGameStatus();
            return;
        }

        // Hide keyboard
        InputMethodManager im =  (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        im.hideSoftInputFromWindow(this.getCurrentFocus().getWindowToken(), InputMethodManager.HIDE_NOT_ALWAYS);

        // You won!
        WinDialog dialog = new WinDialog(this, new WinDialogEventListener(currentGame));
        dialog.show();

        // Quit game and fix underlying UI
        SaveGame();
        currentGame.quit();
        UpdateGameStatus();
    }

    public void ShowHint()
    {
        int suggestedNumber = currentGame.provideHint();

        // Update game status
        UpdateGameStatus();

        // Fix UI
        String numberAsText = String.format("%d", suggestedNumber);
        NumberToTryRef.setText(numberAsText);
    }

    public void QuitGame()
    {
        String msg = String.format("%s %d",
                getString(R.string.theNumberWas),
                currentGame.getSecretNumber());
        GameActivityInitializer.MessageBox(this, msg);

        // Quit and update the status of the game
        currentGame.quit();
        UpdateGameStatus();
    }
    
    private void RefreshView()
    {
        // Set player name and level
        PlayerLabelRef.setText(currentGame.Details.PlayerName);
        LevelLabelRef.setText(currentGame.Details.Level.toString());

        // Update the status of the game
        UpdateGameStatus();
    }

    private void UpdateGameStatus()
    {
        // Set current number of attempts
        String attemptsTemplate = "<b>%d</b> <small>%s</small>";
        String attemptsMarkup = String.format(attemptsTemplate,
                currentGame.Attempts,
                getString(R.string.attemptsLabel));
        AttemptsLabelRef.setText(Html.fromHtml(attemptsMarkup));

        // Set current number of hints
        String hintsTemplate = "<b>%d</b> <small>%s</small>";
        String hintsMarkup = String.format(hintsTemplate,
                currentGame.HintsRequested,
                getString(R.string.hintsLabel));
        HintsLabelRef.setText(Html.fromHtml(hintsMarkup));

        // Set current range
        String rangeTemplate = "<b>%s </b> <small><font color='#00ffff'>%,d</font></small>" +
                "<b> %s </b> <small><font color='#00ffff'>%,d</font></small>";
        String rangeMarkup = String.format(rangeTemplate,
                getString(R.string.betweenLabel), currentGame.Lowest,
                getString(R.string.andLabel), currentGame.Highest);
        StatusLabelRef.setText(Html.fromHtml(rangeMarkup));
        
        // Clear edit box
        NumberToTryRef.setText("");
    }

    private void SaveGame()
    {
        ScoreInfo info = new ScoreInfo();
        info.Name = currentGame.Details.PlayerName;
        info.DateOfScore = currentGame.GameDate.toLocaleString();
        info.Points = currentGame.Score;

        GuessHistory history = PreferenceHelper.LoadHistory(this);
        history.Scores.add(info);
        PreferenceHelper.SaveHistory(this, history);
    }
}
