using System;
using System.IO;
using System.Web;
using WURFL;
using WURFL.Config;
using WurflPeek.ViewModels;

namespace WurflPeek.Framework.WurflHelpers
{
    public class WurflHelper
    {
        /// <summary>
        /// Internal name used to cache the manager
        /// </summary>
        private const String WurflManagerCacheKey = "__WurflManager_v1_4";

        /// <summary>
        /// Gets the currently cached instance of the WURFL manager object.
        /// </summary>
        public static IWURFLManager Manager
        {
            get { return HttpContext.Current.Cache[WurflManagerCacheKey] as IWURFLManager; }
        }

        /// <summary>
        /// To be called once in the app, loads the WURFL database in memory.
        /// </summary>
        public static void Initialize()
        {
            const String wurflDataFilePath = "~/App_Data/wurfl-latest.zip";
            const String wurflPatchFilePath = "~/App_Data/web_browsers_patch.xml";

            var wurflDataFile = HttpContext.Current.Server.MapPath(wurflDataFilePath);
            var wurflPatchFile = HttpContext.Current.Server.MapPath(wurflPatchFilePath);
            var configurer = new InMemoryConfigurer()
                    .MainFile(wurflDataFile)
                    .PatchFile(wurflPatchFile);
            var manager = WURFLManagerBuilder.Build(configurer);
            HttpContext.Current.Cache[WurflManagerCacheKey] = manager;
        }

        /// <summary>
        /// Modifies the view name adding a "mobile-related" suffix as per detected capabilities.
        /// </summary>
        /// <param name="userAgent"></param>
        /// <param name="viewName">Name of the view</param>
        /// <returns>New name of the view</returns>
        public static String TransformViewName(String userAgent, String viewName)
        {
            var deviceInfo = WurflHelper.Manager.GetDeviceForRequest(userAgent);
            var resolvedName = viewName; 

            // Is it a smartphone?
            if (WurflHelper.IsSmartphone(deviceInfo))
                resolvedName = GetModifiedUrl(viewName, "smartphone");
            else
            {
                // Is it just wireless?
                if (WurflHelper.IsMobile(deviceInfo))
                    resolvedName = GetModifiedUrl(viewName, "mobile");
            }
            return resolvedName;
        }

        public static CapsViewModel GrabCommonCapabilities(HttpRequestBase request)
        {
            var device = WurflHelper.Manager.GetDeviceForRequest(request.UserAgent);
            var model = new CapsViewModel();

            #region Browser
            for(var i=0; i<request.Headers.Count; i++)
            {
                model.Browser.Add(new Capability { Name = request.Headers.GetKey(i), Value = request.Headers.Get(i) });
            }

            #endregion

            #region Device
            var deviceModel = String.Format("{0} {1} {2}",
                    device.GetCapability("brand_name"),
                    device.GetCapability("model_name"),
                    device.GetCapability("marketing_name"));
            model.Device.Add(new Capability { Name = "Model", Value = deviceModel });

            var deviceDate = device.GetCapability("release_date").ToNiceDate();
            model.Device.Add(new Capability { Name = "Released", Value = deviceDate });

            var deviceOs = String.Format("{0} {1}",
                device.GetCapability("device_os"),
                device.GetCapability("device_os_version"));
            model.Device.Add(new Capability { Name = "OS", Value = deviceOs });

            var browser = String.Format("{0} {1}",
                device.GetCapability("mobile_browser"),
                device.GetCapability("mobile_browser_version"));
            model.Device.Add(new Capability { Name = "Browser", Value = browser });

            var extra = device.GetCapability("is_tablet").ToBool() ? "Tablet" : "No tablet";
            model.Device.Add(new Capability { Name = "Type", Value = extra });

            var radio = device.GetCapability("has_cellular_radio");
            model.Device.Add(new Capability { Name = "SIM support", Value = radio });
            #endregion

            #region Page
            var markup = device.GetCapability("preferred_markup");
            model.Page.Add(new Capability { Name = "Markup", Value = markup });
            var cookie = device.GetCapability("cookie_support");
            model.Page.Add(new Capability { Name = "Cookie", Value = cookie });
            var landscape = device.GetCapability("dual_orientation");
            model.Page.Add(new Capability { Name = "Landscape", Value = landscape });
            var screen = String.Format("{0} x {1}",
                device.GetCapability("resolution_width"),
                device.GetCapability("resolution_height"));
            model.Page.Add(new Capability { Name = "Screen", Value = screen });
            var inlining = device.GetCapability("image_inlining");
            model.Page.Add(new Capability { Name = "Inline images", Value = inlining });
            #endregion

            #region Script
            var wifi = device.GetCapability("wifi");
            model.Script.Add(new Capability { Name = "Wifi", Value = wifi });
            var ajax = device.GetCapability("ajax_xhr_type");
            model.Script.Add(new Capability { Name = "Ajax", Value = ajax });
            var js = device.GetCapability("ajax_support_javascript");
            model.Script.Add(new Capability { Name = "Javascript", Value = js });
            var cssgrad = device.GetCapability("css_gradient");
            model.Script.Add(new Capability { Name = "CSS gradient", Value = cssgrad });
            #endregion

            return model;
        }

        public static MqViewModel GrabSomeData(HttpRequestBase request)
        {
            return new MqViewModel
                       {
                           Text = "This is some text to display properly",
                           UserAgent = request.UserAgent
                       };
        }

        #region Private members
        /// <summary>
        /// Indicates whether the browser behind the current request is recognized as a smartphone.
        /// </summary>
        /// <param name="device">WURFL device</param>
        /// <returns>True if it's a smartphone, false otherwise</returns>
        private static Boolean IsSmartphone(IDevice device)
        {
            // Must be wireless
            if (!device.IsWireless())
                return false;

            // Must be touch
            if (!device.IsTouch())
                return false;

            // Must be 320px wide or more
            if (device.Width() < 320)
                return false;

            // Must be Android 2+ or iPhone OS
            if (device.HasOs("android", new Version(2, 0)))
                return true;
            if (device.HasOs("iphone os"))
                return true;

            return false;
        }


        /// <summary>
        /// Indicates whether the browser behind the current request is recognized as a mobile device.
        /// </summary>
        /// <param name="device">WURFL device</param>
        /// <returns>True if it's a plain mobile device, false otherwise</returns>
        private static Boolean IsMobile(IDevice device)
        {
            // Must be wireless
            return device.IsWireless();
        }

        /// <summary>
        /// Resolve the resource name applying mobile suffix
        /// </summary>
        /// <param name="name">Original name</param>
        /// <param name="suffix">Suffix to add</param>
        /// <returns>Modified view name</returns>
        private static String GetModifiedUrl(String name, String suffix)
        {
            var mobileExt = String.Format("{0}{1}", suffix, Path.GetExtension(name));
            var url = Path.ChangeExtension(name, mobileExt);
            return url;
        } 
        #endregion
    }
}