//Remember to reference ServiceModel.Web and Runtime.Serialization
using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;


namespace Expoware.Utilities
{
    /// <summary>
    /// Base class for persisting application settings to a fixed iso-store file.
    /// </summary>
    /// <typeparam name="T">Type of the object to persist</typeparam>
    public class SettingsContainer<T> where T:new()
    {
        protected static readonly String FileName = "scorelist.dat";

        public SettingsContainer()
        {
            Data = new T();
        }

        #region Public API 

        public T Data { get; private set; }

        /// <summary>
        /// Save data to the storage
        /// </summary>
        /// <returns>True if successful, False otherwise</returns>
        public Boolean Save()
        {
            return SaveSettingsInternal(FileName, Data);
        }

        /// <summary>
        /// Load data from storage
        /// </summary>
        public T Load()
        {
            Data = ReadSettingsInternal(FileName);
            return Data;
        }
        #endregion

        #region Internals
        protected virtual Boolean SaveSettingsInternal(String fileName, T data)
        {
            // Iso store reference
            var isoStore = default(IsolatedStorageFile);

            // Result of the whole operation
            var result = false;

            // Create the serializer object
            var serializer = new DataContractJsonSerializer(typeof(T));

            try
            {
                using (isoStore = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    using (var targetFile = isoStore.OpenFile(fileName, FileMode.Create))
                    {
                        serializer.WriteObject(targetFile, data);
                        result = true;
                    }
                }
            }
            catch (InvalidDataContractException)
            {
                if (isoStore != null)
                    isoStore.DeleteFile(fileName);
            }
            catch (SerializationException)
            {
                if (isoStore != null)
                    isoStore.DeleteFile(fileName);
            }
            catch
            {
                // Possible exceptions are: IsolatedStorageException|DirectoryNotFound
            }

            // Returns outcome of the operation
            return result;
        }

        protected virtual T ReadSettingsInternal(String fileName)
        {
            // Iso store reference
            var isoStore = default(IsolatedStorageFile);

            // Initialize the object to return.
            var state = default(T);

            // Create the serializer object.
            var serializer = new DataContractJsonSerializer(typeof(T));

            // Grabs handle to the isolated storage, opens the file (if existing) and reads data.
            try
            {
                using (isoStore = IsolatedStorageFile.GetUserStoreForApplication())
                {
                    if (isoStore.FileExists(fileName))
                    {
                        using (var stream = isoStore.OpenFile(fileName, FileMode.Open))
                        {
                            state = (T) serializer.ReadObject(stream);
                        }
                    }
                }
            }
            catch (NullReferenceException)
            {
            }
            catch (IsolatedStorageException)
            {
                if (isoStore != null)
                    isoStore.DeleteFile(fileName);
            }

            // Returns data.
            return state;
        }
        #endregion
    }
}